@@ -1,7 +1,7 @@
 package Twitter::API;
 # ABSTRACT: A Twitter REST API library for Perl
 
-our $VERSION = '0.0106';
+our $VERSION = '0.0107';
 use 5.14.1;
 use Moo;
 use Carp;
@@ -422,9 +422,12 @@ __END__
 
 Twitter::API - A Twitter REST API library for Perl
 
+=for html <a href="http://github.com/badges/stability-badges"><img src="http://badges.github.io/stability-badges/dist/experimental.svg" alt="experimental" /></a>
+<a href="http://travis-ci.org/semifor/Twitter-API"><img src="https://secure.travis-ci.org/semifor/Twitter-API.svg" alt="Build Status" /></a>
+
 =head1 VERSION
 
-version 0.0106
+version 0.0107
 
 =head1 SYNOPSIS
 
@@ -681,16 +684,24 @@ C<password>.
 
 =item *
 
+L<API::Twitter> - Twitter.com API Client
+
+=item *
+
+L<AnyEvent::Twitter::Stream> - Receive Twitter streaming API in an event loop
+
+=item *
+
+L<AnyEvent::Twitter> - A thin wrapper for Twitter API using OAuth
+
+=item *
+
+L<Mojo::WebService::Twitter> - Simple Twitter API client
+
+=item *
+
 L<Net::Twitter> - Twitter::API's predecessor (also L<Net::Twitter::Lite>)
 
-=item *
-
-L<Mojo::WebService::Twitter> - Simple non-blocking Twitter API client
-
-=item *
-
-L<API::Twitter> - Another simple Twitter API client
-
 =back
 
 =head1 AUTHOR
