@@ -1,12 +1,16 @@
 package Twitter::API::Trait::ApiMethods;
 # ABSTRACT: Convenient API Methods
-$Twitter::API::Trait::ApiMethods::VERSION = '0.0106';
+$Twitter::API::Trait::ApiMethods::VERSION = '0.0107';
 use 5.14.1;
 use Carp;
 use Moo::Role;
 use MooX::Aliases;
 use namespace::clean;
 
+requires 'request';
+
+with 'Twitter::API::Role::RequestArgs';
+
 #pod =method account_settings([ \%args ])
 #pod
 #pod L<https://dev.twitter.com/rest/reference/get/account/settings>
@@ -50,7 +54,7 @@ alias blocks_ids => 'blocking_ids';
 #pod =cut
 
 sub collection_entries {
-    shift->_with_pos_args(id => get => 'collections/entries', @_);
+    shift->request_with_pos_args(id => get => 'collections/entries', @_);
 }
 
 #pod =method collections([ $screen_name | $user_id, ][ \%args ])
@@ -60,7 +64,7 @@ sub collection_entries {
 #pod =cut
 
 sub collections {
-    shift->_with_pos_args(':ID', get => 'collections/list', @_);
+    shift->request_with_pos_args(':ID', get => 'collections/list', @_);
 }
 
 #pod =method direct_messages([ \%args ])
@@ -103,7 +107,7 @@ alias followers_list => 'followers';
 #pod =cut
 
 sub followers_ids {
-    shift->_with_pos_args(':ID', get => 'followers/ids', @_);
+    shift->request_with_pos_args(':ID', get => 'followers/ids', @_);
 }
 
 #pod =method friends([ \%args ])
@@ -128,7 +132,7 @@ alias friends_list => 'friends';
 #pod =cut
 
 sub friends_ids {
-    shift->_with_optional_id(get => 'friends/ids', @_);
+    shift->request_with_id(get => 'friends/ids', @_);
 }
 alias following_ids => 'friends_ids';
 
@@ -166,7 +170,7 @@ alias outgoing_friendships => 'friendships_outgoing';
 
 # NT incompatibility
 sub geo_id {
-    shift->_with_pos_args(place_id => get => 'geo/id/:place_id', @_);
+    shift->request_with_pos_args(place_id => get => 'geo/id/:place_id', @_);
 }
 
 #pod =method geo_search([ \%args ])
@@ -335,7 +339,7 @@ sub lookup_friendships {
 #pod =cut
 
 sub lookup_statuses {
-    shift->_with_pos_args(id => get => 'statuses/lookup', @_);
+    shift->request_with_pos_args(id => get => 'statuses/lookup', @_);
 }
 
 #pod =method lookup_users([ \%args ])
@@ -437,7 +441,7 @@ sub rate_limit_status {
 #pod =cut
 
 sub retweeters_ids {
-    shift->_with_pos_args(id => get => 'statuses/retweeters/ids', @_);
+    shift->request_with_pos_args(id => get => 'statuses/retweeters/ids', @_);
 }
 
 #pod =method retweets([ $id, ][ \%args ])
@@ -447,7 +451,7 @@ sub retweeters_ids {
 #pod =cut
 
 sub retweets {
-    shift->_with_pos_args(id => get => 'statuses/retweets/:id', @_);
+    shift->request_with_pos_args(id => get => 'statuses/retweets/:id', @_);
 }
 
 #pod =method retweets_of_me([ \%args ])
@@ -470,7 +474,7 @@ alias retweeted_of_me => 'retweets_of_me';
 #pod =cut
 
 sub reverse_geocode {
-    shift->_with_pos_args([ qw/lat long/ ], get => 'geo/reverse_geocode', @_);
+    shift->request_with_pos_args([ qw/lat long/ ], get => 'geo/reverse_geocode', @_);
 }
 
 #pod =method saved_searches([ \%args ])
@@ -490,7 +494,7 @@ sub saved_searches {
 #pod =cut
 
 sub search {
-    shift->_with_pos_args(q => get => 'search/tweets', @_);
+    shift->request_with_pos_args(q => get => 'search/tweets', @_);
 }
 
 #pod =method sent_direct_messages([ \%args ])
@@ -513,7 +517,7 @@ alias direct_messages_sent => 'sent_direct_messages';
 #pod =cut
 
 sub show_direct_message {
-    shift->_with_pos_args(id => get => 'direct_messages/show', @_);
+    shift->request_with_pos_args(id => get => 'direct_messages/show', @_);
 }
 
 #pod =method show_friendship([ \%args ])
@@ -562,7 +566,7 @@ alias $_ => 'show_list_subscriber' for qw/is_list_subscriber is_subscriber_lists
 #pod =cut
 
 sub show_saved_search {
-    shift->_with_pos_args(id => get => 'saved_searches/show/:id', @_);
+    shift->request_with_pos_args(id => get => 'saved_searches/show/:id', @_);
 }
 
 #pod =method show_status([ $id, ][ \%args ])
@@ -572,7 +576,7 @@ sub show_saved_search {
 #pod =cut
 
 sub show_status {
-    shift->_with_pos_args(id => get => 'statuses/show/:id', @_);
+    shift->request_with_pos_args(id => get => 'statuses/show/:id', @_);
 }
 
 #pod =method show_user([ $screen_name | $user_id, ][ \%args ])
@@ -582,7 +586,7 @@ sub show_status {
 #pod =cut
 
 sub show_user {
-    shift->_with_pos_args(':ID', get => 'users/show', @_);
+    shift->request_with_pos_args(':ID', get => 'users/show', @_);
 }
 
 #pod =method suggestion_categories([ \%args ])
@@ -626,7 +630,7 @@ sub trends_closest {
 #pod =cut
 
 sub trends_place {
-    shift->_with_pos_args(id => get => 'trends/place', @_);
+    shift->request_with_pos_args(id => get => 'trends/place', @_);
 }
 alias trends_location => 'trends_place';
 
@@ -648,7 +652,7 @@ my $rename_category = sub {
 sub user_suggestions {
     my $self = shift;
 
-    $self->_with_pos_args(slug => get => 'users/suggestions/:slug/members',
+    $self->request_with_pos_args(slug => get => 'users/suggestions/:slug/members',
         $self->$rename_category(@_));
 }
 alias follow_suggestions => 'user_suggestions';
@@ -664,7 +668,7 @@ alias follow_suggestions => 'user_suggestions';
 sub user_suggestions_for {
     my $self = shift;
 
-    $self->_with_pos_args(slug => get => 'users/suggestions/:slug',
+    $self->request_with_pos_args(slug => get => 'users/suggestions/:slug',
         $self->$rename_category(@_));
 }
 alias follow_suggestions_for => 'user_suggestions_for';
@@ -676,7 +680,7 @@ alias follow_suggestions_for => 'user_suggestions_for';
 #pod =cut
 
 sub user_timeline {
-    shift->_with_optional_id(get => 'statuses/user_timeline', @_);
+    shift->request_with_id(get => 'statuses/user_timeline', @_);
 }
 
 #pod =method users_search([ $q, ][ \%args ])
@@ -688,7 +692,7 @@ sub user_timeline {
 #pod =cut
 
 sub users_search {
-    shift->_with_pos_args(q => get => 'users/search', @_);
+    shift->request_with_pos_args(q => get => 'users/search', @_);
 }
 alias $_ => 'users_search' for qw/find_people search_users/;
 
@@ -709,7 +713,7 @@ sub verify_credentials {
 #pod =cut
 
 sub add_collection_entry {
-    shift->_with_pos_args([ qw/id tweet_id /],
+    shift->request_with_pos_args([ qw/id tweet_id /],
         post => 'collections/entries/add', @_);
 }
 
@@ -725,7 +729,7 @@ sub add_list_member {
 
 # deprecated: https://dev.twitter.com/rest/reference/post/geo/place
 sub add_place {
-    shift->_with_pos_args([ qw/name contained_within token lat long/ ],
+    shift->request_with_pos_args([ qw/name contained_within token lat long/ ],
         post => 'geo/place', @_);
 }
 
@@ -736,7 +740,7 @@ sub add_place {
 #pod =cut
 
 sub create_block {
-    shift->_with_pos_args(':ID', post => 'blocks/create', @_);
+    shift->request_with_pos_args(':ID', post => 'blocks/create', @_);
 }
 
 #pod =method create_collection([ $name, ][ \%args ])
@@ -746,7 +750,7 @@ sub create_block {
 #pod =cut
 
 sub create_collection {
-    shift->_with_pos_args(name => post => 'collections/create', @_);
+    shift->request_with_pos_args(name => post => 'collections/create', @_);
 }
 
 #pod =method create_favorite([ $id, ][ \%args ])
@@ -756,7 +760,7 @@ sub create_collection {
 #pod =cut
 
 sub create_favorite {
-    shift->_with_pos_args(id => post => 'favorites/create', @_);
+    shift->request_with_pos_args(id => post => 'favorites/create', @_);
 }
 
 #pod =method create_friend([ $screen_name | $user_id, ][ \%args ])
@@ -768,7 +772,7 @@ sub create_favorite {
 #pod =cut
 
 sub create_friend {
-    shift->_with_pos_args(':ID', post => 'friendships/create', @_);
+    shift->request_with_pos_args(':ID', post => 'friendships/create', @_);
 }
 alias $_ => 'create_friend' for qw/follow follow_new create_friendship/;
 
@@ -779,7 +783,7 @@ alias $_ => 'create_friend' for qw/follow follow_new create_friendship/;
 #pod =cut
 
 sub create_list {
-    shift->_with_pos_args(name => post => 'lists/create', @_);
+    shift->request_with_pos_args(name => post => 'lists/create', @_);
 }
 
 #pod =method create_media_metadata([ \%args ])
@@ -808,7 +812,7 @@ sub create_media_metadata {
 #pod =cut
 
 sub create_mute {
-    shift->_with_pos_args(id => post => 'mutes/users/create', @_);
+    shift->request_with_pos_args(id => post => 'mutes/users/create', @_);
 }
 
 #pod =method create_saved_search([ $query, ][ \%args ])
@@ -818,7 +822,7 @@ sub create_mute {
 #pod =cut
 
 sub create_saved_search {
-    shift->_with_pos_args(query => post => 'saved_searches/create', @_);
+    shift->request_with_pos_args(query => post => 'saved_searches/create', @_);
 }
 
 #pod =method curate_collection([ \%args ])
@@ -864,7 +868,7 @@ alias remove_list_member => 'delete_list_member';
 #pod =cut
 
 sub destroy_block {
-    shift->_with_pos_args(':ID', post => 'blocks/destroy', @_);
+    shift->request_with_pos_args(':ID', post => 'blocks/destroy', @_);
 }
 
 #pod =method destroy_collection([ $id, ][ \%args ])
@@ -874,7 +878,7 @@ sub destroy_block {
 #pod =cut
 
 sub destroy_collection {
-    shift->_with_pos_args(id => post => 'collections/destroy', @_);
+    shift->request_with_pos_args(id => post => 'collections/destroy', @_);
 }
 
 #pod =method destroy_direct_message([ $id, ][ \%args ])
@@ -884,7 +888,7 @@ sub destroy_collection {
 #pod =cut
 
 sub destroy_direct_message {
-    shift->_with_pos_args(id => post => 'direct_messages/destroy', @_);
+    shift->request_with_pos_args(id => post => 'direct_messages/destroy', @_);
 }
 
 #pod =method destroy_favorite([ $id, ][ \%args ])
@@ -894,7 +898,7 @@ sub destroy_direct_message {
 #pod =cut
 
 sub destroy_favorite {
-    shift->_with_pos_args(id => post => 'favorites/destroy', @_);
+    shift->request_with_pos_args(id => post => 'favorites/destroy', @_);
 }
 
 #pod =method destroy_friend([ $screen_name | $user_id, ][ \%args ])
@@ -906,7 +910,7 @@ sub destroy_favorite {
 #pod =cut
 
 sub destroy_friend {
-    shift->_with_pos_args(':ID', post => 'friendships/destroy', @_);
+    shift->request_with_pos_args(':ID', post => 'friendships/destroy', @_);
 }
 alias $_ => 'destroy_friend' for qw/unfollow destroy_friendship/;
 
@@ -917,7 +921,7 @@ alias $_ => 'destroy_friend' for qw/unfollow destroy_friendship/;
 #pod =cut
 
 sub destroy_mute {
-    shift->_with_pos_args(id => post => 'mutes/users/destroy', @_);
+    shift->request_with_pos_args(id => post => 'mutes/users/destroy', @_);
 }
 
 #pod =method destroy_saved_search([ $id, ][ \%args ])
@@ -929,7 +933,7 @@ sub destroy_mute {
 #pod =cut
 
 sub destroy_saved_search {
-    shift->_with_pos_args(id => post => 'saved_searches/destroy/:id', @_);
+    shift->request_with_pos_args(id => post => 'saved_searches/destroy/:id', @_);
 }
 alias delete_saved_search => 'destroy_saved_search';
 
@@ -940,7 +944,7 @@ alias delete_saved_search => 'destroy_saved_search';
 #pod =cut
 
 sub destroy_status {
-    shift->_with_pos_args(id => post => 'statuses/destroy/:id', @_);
+    shift->request_with_pos_args(id => post => 'statuses/destroy/:id', @_);
 }
 
 #pod =method members_create_all([ \%args ])
@@ -976,7 +980,7 @@ alias remove_list_members => 'members_destroy_all';
 #pod =cut
 
 sub move_collection_entry {
-    shift->_with_pos_args([ qw/id tweet_id relative_to /],
+    shift->request_with_pos_args([ qw/id tweet_id relative_to /],
         post => 'collections/entries/move', @_);
 }
 
@@ -987,7 +991,7 @@ sub move_collection_entry {
 #pod =cut
 
 sub new_direct_message {
-    shift->_with_pos_args([ qw/text :ID/ ], post => 'direct_messages/new', @_);
+    shift->request_with_pos_args([ qw/text :ID/ ], post => 'direct_messages/new', @_);
 }
 
 #pod =method remove_collection_entry([ $id, [ $tweet_id, ]][ \%args ])
@@ -997,7 +1001,7 @@ sub new_direct_message {
 #pod =cut
 
 sub remove_collection_entry {
-    shift->_with_pos_args([ qw/id tweet_id/ ],
+    shift->request_with_pos_args([ qw/id tweet_id/ ],
         post => 'collections/entries/remove', @_);
 }
 
@@ -1018,7 +1022,7 @@ sub remove_profile_banner {
 #pod =cut
 
 sub report_spam {
-    shift->_with_optional_id(post => 'users/report_spam', @_);
+    shift->request_with_id(post => 'users/report_spam', @_);
 }
 
 #pod =method retweet([ $id, ][ \%args ])
@@ -1028,7 +1032,7 @@ sub report_spam {
 #pod =cut
 
 sub retweet {
-    shift->_with_pos_args(id => post => 'statuses/retweet/:id', @_);
+    shift->request_with_pos_args(id => post => 'statuses/retweet/:id', @_);
 }
 
 #pod =method subscribe_list([ \%args ])
@@ -1048,7 +1052,7 @@ sub subscribe_list {
 #pod =cut
 
 sub unretweet {
-    shift->_with_pos_args(id => post => 'statuses/unretweet/:id', @_);
+    shift->request_with_pos_args(id => post => 'statuses/unretweet/:id', @_);
 }
 
 #pod =method unsubscribe_list([ \%args ])
@@ -1068,7 +1072,7 @@ sub unsubscribe_list {
 #pod =cut
 
 sub update {
-    shift->_with_pos_args(status => post => 'statuses/update', @_);
+    shift->request_with_pos_args(status => post => 'statuses/update', @_);
 }
 
 #pod =method update_account_settings([ \%args ])
@@ -1088,7 +1092,7 @@ sub update_account_settings {
 #pod =cut
 
 sub update_collection {
-    shift->_with_pos_args(id => post => 'collections/update', @_);
+    shift->request_with_pos_args(id => post => 'collections/update', @_);
 }
 
 #pod =method update_friendship([ \%args ])
@@ -1098,7 +1102,7 @@ sub update_collection {
 #pod =cut
 
 sub update_friendship {
-    shift->_with_optional_id(post => 'friendships/update', @_);
+    shift->request_with_id(post => 'friendships/update', @_);
 }
 
 #pod =method update_list([ \%args ])
@@ -1138,7 +1142,7 @@ sub update_profile_background_image {
 #pod =cut
 
 sub update_profile_banner {
-    shift->_with_pos_args(banner => post => 'account/update_profile_banner', @_);
+    shift->request_with_pos_args(banner => post => 'account/update_profile_banner', @_);
 }
 
 #pod =method update_profile_image([ $image, ][ \%args ])
@@ -1148,7 +1152,7 @@ sub update_profile_banner {
 #pod =cut
 
 sub update_profile_image {
-    shift->_with_pos_args(image => post => 'account/update_profile_image', @_);
+    shift->request_with_pos_args(image => post => 'account/update_profile_image', @_);
 }
 
 #pod =method upload_media([ $media, ][ \%args ])
@@ -1185,59 +1189,6 @@ sub upload_media {
 }
 alias upload => 'upload_media';
 
-
-# if there is a positional arg, it's an :ID (screen_name or user_id)
-sub _with_optional_id {
-    splice @_, 1, 0, [];
-    push @{$_[1]}, ':ID' if @_ > 4 && ref $_[4] ne 'HASH';
-    goto $_[0]->can('_with_pos_args');
-}
-
-sub _with_pos_args {
-    my $self        = shift;
-    my @pos_names   = shift;
-    my $http_method = shift;
-    my $path        = shift;
-    my %args;
-
-    # names can be a single value or an arrayref
-    @pos_names = @{ $pos_names[0] } if ref $pos_names[0] eq 'ARRAY';
-
-    # gather positional arguments and name them
-    while ( @pos_names ) {
-        last if @_ == 0 || ref $_[0] eq 'HASH';
-        $args{shift @pos_names} = shift;
-    }
-
-    # get the optional, following args hashref and expand it
-    my %args_hash; %args_hash = %{ shift() } if ref $_[0] eq 'HASH';
-
-    # extract any required args if we still have names
-    while ( my $name = shift @pos_names ) {
-        if ( $name eq ':ID' ) {
-            $name = exists $args_hash{screen_name} ? 'screen_name' : 'user_id';
-            croak 'missing required screen_name or user_id'
-                unless exists $args_hash{$name};
-        }
-        croak "missing required '$name' arg" unless exists $args_hash{$name};
-        $args{$name} = delete $args_hash{$name};
-    }
-
-    # name the :ID value (if any) based on its value
-    if ( my $id = delete $args{':ID'} ) {
-        $args{$id =~/\D/ ? 'screen_name' : 'user_id'} = $id;
-    }
-
-    # merge in the remaining optional values
-    for my $name ( keys %args_hash ) {
-        croak "'$name' specified in both positional and named args"
-            if exists $args{$name};
-        $args{$name} = $args_hash{$name};
-    }
-
-    $self->request($http_method, $path, \%args, @_);
-}
-
 1;
 
 __END__
@@ -1252,7 +1203,7 @@ Twitter::API::Trait::ApiMethods - Convenient API Methods
 
 =head1 VERSION
 
-version 0.0106
+version 0.0107
 
 =head1 DESCRIPTION
 
